@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2014-2015 -- leonerd@leonerd.org.uk
 
 package Device::BusPirate::Mode::I2C;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Device::BusPirate::Mode );
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 use Carp;
 
@@ -228,6 +228,42 @@ sub recv
    });
 }
 
+=head2 $bytes_in = $ic->send_then_recv( $address, $bytes_out, $read_len )->get
+
+A convenient wrapper around C<start_bit>, C<write>, C<read> and C<stop_bit>.
+This method combines a C<send> and C<recv> operation, with a repeated START
+condition inbetween (not a STOP). It is useful when reading values from I2C
+slaves that implement numbered registers; sending the register number as a
+write, before requesting the read.
+
+C<$address> should be an integer, in the range 0 to 0x7f.
+
+=cut
+
+sub send_then_recv
+{
+   my $self = shift;
+   my ( $address, $bytes_out, $read_len ) = @_;
+
+   $address >= 0 and $address < 0x80 or
+      croak "Invalid I2C slave address";
+
+   $self->pirate->enter_mutex( sub {
+      $self->start_bit->then( sub {
+         $self->write( chr( $address << 1 | 0 ) . $bytes_out )
+      })->then( sub {
+         $self->start_bit # repeated START
+      })->then( sub {
+         $self->write( chr( $address << 1 | 1 ) )
+      })->then( sub {
+         $self->read( $read_len )
+      })->then( sub {
+         my ( $bytes_in ) = @_;
+         $self->stop_bit->then_done( $bytes_in );
+      });
+   });
+}
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
